// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef SANITIZE_TEXT_DOCUMENT_H
#define SANITIZE_TEXT_DOCUMENT_H

class QTextDocument;

void sanitizeTextDocument(QTextDocument *document);

#endif // SANITIZE_TEXT_DOCUMENT_H
