// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef PREDEFINEDCOMMANDS_H
#define PREDEFINEDCOMMANDS_H

#include "common/command.h"

#include <QtContainerFwd>

QVector<Command> predefinedCommands();

#endif // PREDEFINEDCOMMANDS_H
